@@ -16,13 +16,14 @@ class Container(Component):
     The Container loads and instantiates components.
     """
     
-    def __init__(self, *paths):
+    def __init__(self, paths, blacklist = []):
         """
         @param paths: paths where to look for components
         """
     
         self.__components = []
         self.__devices = []
+        self.__blacklist = blacklist
         
         Component.__init__(self)
         
@@ -30,6 +31,13 @@ class Container(Component):
             if (os.path.exists(p)):
                 self.load_path(p)
 
+        for c in self.__components:
+            self.emit_event(msgs.COM_EV_COMPONENT_LOADED, c)
+
+        for dev in self.__devices:
+            self.emit_event(msgs.CORE_EV_DEVICE_ADDED, dev.get_device_id(), dev)
+
+
         
     def __find_modules(self, path):
         """
@@ -46,6 +54,8 @@ class Container(Component):
             dirs = ["core"] + dirs
 
         for f in dirs:
+            if (f in self.__blacklist): continue
+            
             comppath = os.path.join(path, f)
             if (os.path.isdir(comppath) and 
                   not f.startswith(".")):
@@ -152,8 +162,8 @@ class Container(Component):
         @param path: path of components directory
         """
         
-        self.__components = []
-        self.__devices = []
+        #self.__components = []
+        #self.__devices = []
         
         mods = self.__find_modules(path)
         for mod in mods:
@@ -162,9 +172,3 @@ class Container(Component):
             self.emit_event(msgs.COM_EV_LOADING_MODULE, mod.__name__)
             self.__load_components(mod)
 
-        for c in self.__components:
-            self.emit_event(msgs.COM_EV_COMPONENT_LOADED, c)
-
-        for dev in self.__devices:
-            self.emit_event(msgs.CORE_EV_DEVICE_ADDED, dev.get_device_id(), dev)
-
