@@ -12,8 +12,11 @@ class DeviceThumbnail(Thumbnail):
     def __init__(self, dev):
 
         title = dev.get_name()
-        thumb = dev.get_icon() or theme.mb_device_unknown
-        
+        try:
+            thumb = dev.get_icon() or theme.mb_device_unknown
+        except:
+            thumb = theme.mb_device_unknown
+            
         Thumbnail.__init__(self)
         self.set_thumbnail_pbuf(thumb)
         self.set_caption(title)
