@@ -100,6 +100,9 @@ class GenericViewer(Viewer):
         # whether we may advance to the next track
         self.__may_go_next = True
         
+        # table: owner -> needs_reload
+        self.__needs_reload = {}
+        
     
         Viewer.__init__(self)
         
@@ -147,9 +150,8 @@ class GenericViewer(Viewer):
 
         self.accept_device_types(Device.TYPE_GENERIC)
         
-        #self.set_size(800, 480)
-        self.__set_view_mode(self._VIEWMODE_BROWSER)
         gobject.idle_add(self.__init_library)
+        gobject.timeout_add(0, self.__set_view_mode, self._VIEWMODE_BROWSER)
 
 
 
@@ -172,7 +174,7 @@ class GenericViewer(Viewer):
         """
     
         def f():
-            gobject.timeout_add(50, self.__set_view_mode, view_mode)
+            gobject.timeout_add(0, self.__set_view_mode, view_mode)
             
         self.__side_tabs.add_tab(None, name, f)
 
@@ -193,6 +195,7 @@ class GenericViewer(Viewer):
 
         if (self.__is_device_accepted(device)):
             self.__devices[ident] = device
+            self.__strip_needs_reload(self._VIEWMODE_BROWSER)
             self.__update_side_strip()
         
         
@@ -203,6 +206,7 @@ class GenericViewer(Viewer):
     
         try:
             del self.__devices[ident]
+            self.__strip_needs_reload(self._VIEWMODE_BROWSER)
             self.__update_side_strip()
         except:
             pass
@@ -230,6 +234,15 @@ class GenericViewer(Viewer):
         return thumbnails        
 
 
+    def __strip_needs_reload(self, view_mode):
+        """
+        Marks the current side strip to need a reload.
+        """
+        
+        strip_owner = (self, view_mode)
+        self.__needs_reload[strip_owner] = True
+
+
     def __update_side_strip(self):
         """
         Updates the side strip depending on the current view mode.
@@ -237,7 +250,19 @@ class GenericViewer(Viewer):
         
         strip_items = []
         strip_index = -1
-    
+        
+        if (not self.is_active()): return
+        
+        strip_owner = (self, self.__view_mode)
+        needs_reload = self.__needs_reload.get(strip_owner, True)
+        items_to_thumbnail = []
+        
+        if (not needs_reload):            
+            return
+        else:
+            self.__needs_reload[strip_owner] = False
+        
+
         # show devices
         if (self.__view_mode == self._VIEWMODE_BROWSER):
             strip_items = self.__get_device_thumbnails()
@@ -249,10 +274,8 @@ class GenericViewer(Viewer):
         elif (self.__view_mode == self._VIEWMODE_SPLIT_BROWSER):
             if (len(self.__path_stack) > 1):
                 for entry in self.__sibling_folders:
-                    icon, uptodate = self.call_service(
-                                    msgs.MEDIASCANNER_SVC_GET_THUMBNAIL, entry)
-    
-                    tn = FileThumbnail(icon, entry)
+                    tn = FileThumbnail(None, entry)
+                    items_to_thumbnail.append((None, tn, entry))
                     strip_items.append(tn)
                 #end for
                         
@@ -268,16 +291,14 @@ class GenericViewer(Viewer):
         
         # show playable items
         elif (self.__view_mode == self._VIEWMODE_PLAYER_NORMAL):
-            import time
-            print time.time()
             for entry in self.__playable_items:
-                icon, uptodate = self.call_service(
-                                    msgs.MEDIASCANNER_SVC_GET_THUMBNAIL, entry)
-
-                tn = FileThumbnail(icon, entry)
+                thumb = self.call_service(
+                    msgs.MEDIASCANNER_SVC_LOOKUP_THUMBNAIL, entry) or None
+                tn = FileThumbnail(thumb, entry)
+                if (not thumb):
+                    items_to_thumbnail.append((None, tn, entry))
                 strip_items.append(tn)
             #end for
-            print time.time()
             
             if (self.__current_file in self.__playable_items):
                 strip_index = self.__playable_items.index(self.__current_file)           
@@ -286,6 +307,9 @@ class GenericViewer(Viewer):
         if (strip_index >= 0):
             self.hilight_strip_item(strip_index)
 
+        if (self.__path_stack):
+            self.__create_thumbnails(self.__path_stack[-1][0], items_to_thumbnail)
+
 
     def __update_toolbar(self):
         """
@@ -324,19 +348,19 @@ class GenericViewer(Viewer):
         """
         
         if (self.__view_mode == self._VIEWMODE_BROWSER):
-            self.emit_event(msgs.INPUT_EV_CONTEXT_BROWSER)
+            self.emit_message(msgs.INPUT_EV_CONTEXT_BROWSER)
 
         elif (self.__view_mode == self._VIEWMODE_SPLIT_BROWSER):
-            self.emit_event(msgs.INPUT_EV_CONTEXT_BROWSER)
+            self.emit_message(msgs.INPUT_EV_CONTEXT_BROWSER)
 
         elif (self.__view_mode == self._VIEWMODE_PLAYER_NORMAL):
-            self.emit_event(msgs.INPUT_EV_CONTEXT_PLAYER)
+            self.emit_message(msgs.INPUT_EV_CONTEXT_PLAYER)
 
         elif (self.__view_mode == self._VIEWMODE_PLAYER_FULLSCREEN):
-            self.emit_event(msgs.INPUT_EV_CONTEXT_FULLSCREEN)
+            self.emit_message(msgs.INPUT_EV_CONTEXT_FULLSCREEN)
 
         elif (self.__view_mode == self._VIEWMODE_LIBRARY):
-            self.emit_event(msgs.INPUT_EV_CONTEXT_BROWSER)
+            self.emit_message(msgs.INPUT_EV_CONTEXT_BROWSER)
 
 
 
@@ -359,6 +383,11 @@ class GenericViewer(Viewer):
                 idx = self.__playable_items.index(self.__current_file)
                 self.hilight_strip_item(idx)
 
+        elif (self.__view_mode == self._VIEWMODE_PLAYER_FULLSCREEN):
+            if (self.__current_file in self.__playable_items):
+                idx = self.__playable_items.index(self.__current_file)
+                self.hilight_strip_item(idx)
+
     
     
     def render_this(self):
@@ -396,10 +425,14 @@ class GenericViewer(Viewer):
         self.__view_mode = mode
         self.__update_input_context()
         w, h = self.get_size()
+        
+        strip_owner = (self, mode)
+        if (mode != self._VIEWMODE_PLAYER_FULLSCREEN and not was_fullscreen):
+            self.change_strip(strip_owner)
 
         if (mode == self._VIEWMODE_BROWSER):
-            self.emit_event(msgs.UI_ACT_FREEZE)
-            self.emit_event(msgs.UI_ACT_VIEW_MODE, viewmodes.NORMAL)
+            #self.emit_message(msgs.UI_ACT_FREEZE)
+            self.emit_message(msgs.UI_ACT_VIEW_MODE, viewmodes.NORMAL)
             
             #if (self.__path_stack):
             #    self.__load_folder(self.__path_stack[-1][0], None)
@@ -409,18 +442,22 @@ class GenericViewer(Viewer):
             self.__media_box.set_visible(False)
             self.__lib_list.set_visible(False)
 
-            self.__update_side_strip()
-            self.__hilight_current_file()
-            self.__update_toolbar()
+            self.__strip_needs_reload(self._VIEWMODE_BROWSER)
+            if (not was_fullscreen):
+                self.__update_side_strip()
+                self.__hilight_current_file()
+                self.__update_toolbar()
             if (self.__current_device):
                 self.set_title(self.__current_device.get_name())
             
-            self.emit_event(msgs.UI_ACT_THAW)
+            #self.emit_message(msgs.UI_ACT_THAW)
+            #self.render()
+            self.emit_event(msgs.UI_ACT_RENDER)
 
 
         elif (mode == self._VIEWMODE_SPLIT_BROWSER):
-            self.emit_event(msgs.UI_ACT_FREEZE)
-            self.emit_event(msgs.UI_ACT_VIEW_MODE, viewmodes.NORMAL)
+            #self.emit_message(msgs.UI_ACT_FREEZE)
+            self.emit_message(msgs.UI_ACT_VIEW_MODE, viewmodes.NORMAL)
 
             #if (self.__path_stack):
             #    self.__load_folder(self.__path_stack[-1][0], None)
@@ -430,33 +467,40 @@ class GenericViewer(Viewer):
             self.__media_box.set_visible(False)
             self.__lib_list.set_visible(False)
 
-            self.__update_side_strip()
-            self.__hilight_current_file()
-            self.__update_toolbar()
+            if (not was_fullscreen):
+                self.__update_side_strip()
+                self.__hilight_current_file()
+                self.__update_toolbar()
             if (self.__current_device):
                 self.set_title(self.__current_device.get_name())
                         
-            self.emit_event(msgs.UI_ACT_THAW)
+            #self.emit_message(msgs.UI_ACT_THAW)
+            #self.render()
+            self.emit_event(msgs.UI_ACT_RENDER)
 
 
         elif (mode == self._VIEWMODE_PLAYER_NORMAL):
-            self.emit_event(msgs.UI_ACT_VIEW_MODE, viewmodes.NORMAL)
+            self.emit_message(msgs.UI_ACT_VIEW_MODE, viewmodes.NORMAL)
             self.__media_box.set_visible(True)
             self.__side_tabs.set_visible(True)
             self.__list.set_visible(False)
             self.__lib_list.set_visible(False)
+
+            self.__strip_needs_reload(self._VIEWMODE_PLAYER_NORMAL)
+            self.__update_side_strip()
+            self.__hilight_current_file()
             
             if (not was_fullscreen):
-                self.emit_event(msgs.UI_ACT_FREEZE)
-                self.__update_side_strip()
-                self.__hilight_current_file()
+                #self.emit_message(msgs.UI_ACT_FREEZE)
                 self.__update_toolbar()
                 if (self.__current_file):
                     self.set_title(self.__current_file.name)
-                self.emit_event(msgs.UI_ACT_THAW)
-            else:
+                #self.emit_message(msgs.UI_ACT_THAW)
+                #self.render()
+                self.emit_event(msgs.UI_ACT_RENDER)
+            else:                
                 self.render()
-                gobject.timeout_add(50, self.emit_event, msgs.UI_ACT_RENDER)
+                gobject.timeout_add(50, self.emit_message, msgs.UI_ACT_RENDER)
 
 
         elif (mode == self._VIEWMODE_PLAYER_FULLSCREEN):
@@ -465,15 +509,14 @@ class GenericViewer(Viewer):
             self.__lib_list.set_visible(False)
             self.__media_box.set_visible(True)
             self.__media_box.set_geometry(0, 0, 800, 480)
-            self.emit_event(msgs.UI_ACT_VIEW_MODE, viewmodes.FULLSCREEN)
+            self.emit_message(msgs.UI_ACT_VIEW_MODE, viewmodes.FULLSCREEN)
 
-            #self.emit_event(msgs.UI_ACT_THAW)
             self.render()
 
 
         elif (mode == self._VIEWMODE_LIBRARY):           
-            self.emit_event(msgs.UI_ACT_FREEZE)
-            self.emit_event(msgs.UI_ACT_VIEW_MODE, viewmodes.NO_STRIP)
+            #self.emit_message(msgs.UI_ACT_FREEZE)
+            self.emit_message(msgs.UI_ACT_VIEW_MODE, viewmodes.NO_STRIP)
             
             self.__side_tabs.set_visible(True)
             self.__list.set_visible(False)
@@ -483,7 +526,9 @@ class GenericViewer(Viewer):
             self.__update_toolbar()
             self.set_title("Media Library")
             
-            self.emit_event(msgs.UI_ACT_THAW)
+            #self.emit_message(msgs.UI_ACT_THAW)
+            #self.render()
+            self.emit_event(msgs.UI_ACT_RENDER)
        
         
 
@@ -512,7 +557,7 @@ class GenericViewer(Viewer):
             if (self.__current_device and self.__path_stack):
                 #self.__load_device(self.__current_device)
                 path = self.__path_stack[-1][0]
-                self.__load_folder(path, None)
+                self.__load_folder(path, self.__GO_NEW)
         
         # watch for new storage devices
         elif (msg == msgs.CORE_EV_DEVICE_ADDED):
@@ -549,14 +594,14 @@ class GenericViewer(Viewer):
             if (idx != -1):
                 new_idx = min(len(self.__items), idx + 2)
                 self.__list.scroll_to_item(new_idx)
-                self.emit_event(msgs.CORE_ACT_SCROLL_UP)
+                #self.emit_message(msgs.CORE_ACT_SCROLL_UP)
             
         elif (msg == msgs.INPUT_EV_UP):
             idx = self.__list.get_index_at(0)
             if (idx != -1):
                 new_idx = max(0, idx - 2)
                 self.__list.scroll_to_item(new_idx)
-                self.emit_event(msgs.CORE_ACT_SCROLL_DOWN)
+                #self.emit_message(msgs.CORE_ACT_SCROLL_DOWN)
             
         
         # load selected device or file
@@ -567,16 +612,16 @@ class GenericViewer(Viewer):
                 if (dev != self.__current_device):
                     self.__load_device(dev)
 
-            elif (self.__view_mode == self._VIEWMODE_SPLIT_BROWSER):
-                if (len(self.__path_stack) > 1):
-                    folder = self.__sibling_folders[idx]
-                    if (self.__path_stack):
-                        self.__path_stack.pop()
-                    self.__load_folder(folder, None)
-                else:
-                    dev = self.__device_items[idx]
-                    if (dev != self.__current_device):
-                        self.__load_device(dev)               
+            #elif (self.__view_mode == self._VIEWMODE_SPLIT_BROWSER):
+            #    if (len(self.__path_stack) > 1):
+            #        folder = self.__sibling_folders[idx]
+            #        if (self.__path_stack):
+            #            self.__path_stack.pop()
+            #        self.__load_folder(folder, None)
+            #    else:
+            #        dev = self.__device_items[idx]
+            #        if (dev != self.__current_device):
+            #            self.__load_device(dev)               
                     
             elif (self.__view_mode == self._VIEWMODE_PLAYER_NORMAL):
                 item = self.__playable_items[idx]
@@ -593,7 +638,8 @@ class GenericViewer(Viewer):
             dev_id = args[0]
             if (dev_id in self.__devices):
                 self.__load_device(self.__devices[dev_id])
-                self.__update_side_strip()
+                #self.__strip_needs_reload()
+                #self.__update_side_strip()
 
 
         # go to previous
@@ -673,18 +719,19 @@ class GenericViewer(Viewer):
         if (button == item.BUTTON_PLAY):
             entry = self.__items[idx - 1]
             self.__list.hilight(idx)
-            self.__list.render()
-            gobject.timeout_add(50, self.__load_file, entry,
-                                MediaWidget.DIRECTION_NONE)
+            #self.__list.render()
+            #gobject.timeout_add(50, self.__load_file, entry,
+            #                    MediaWidget.DIRECTION_NONE)
+            self.__load_file(entry, MediaWidget.DIRECTION_NONE)
 
         elif (button == item.BUTTON_ENQUEUE):
             if (idx == 0):
                 path, list_offset = self.__path_stack[-1]
-                self.emit_event(msgs.PLAYLIST_ACT_APPEND, path)
+                self.emit_message(msgs.PLAYLIST_ACT_APPEND, path)
 
             else:
                 entry = self.__items[idx - 1]
-                self.emit_event(msgs.PLAYLIST_ACT_APPEND, entry)
+                self.emit_message(msgs.PLAYLIST_ACT_APPEND, entry)
 
         elif (button == item.BUTTON_ADD_TO_LIBRARY):
             path, list_offset = self.__path_stack[-1]
@@ -702,9 +749,11 @@ class GenericViewer(Viewer):
         elif (button == item.BUTTON_OPEN):
             entry = self.__items[idx - 1]
             if (self.__view_mode == self._VIEWMODE_BROWSER):
-                gobject.timeout_add(50, self.__insert_folder, entry)
+                #gobject.timeout_add(50, self.__insert_folder, entry)
+                self.__insert_folder(entry)
             else:
-                gobject.timeout_add(50, self.__load_folder, entry, None)
+                #gobject.timeout_add(50, self.__load_folder, entry, None)
+                self.__load_folder(entry, self.__GO_CHILD)
                 
         elif (button == item.BUTTON_CLOSE):
             self.__close_subfolder()
@@ -758,14 +807,18 @@ class GenericViewer(Viewer):
         self.__path_stack = []
         
         root = device.get_root()
-        print "LOAD DEVICE", device, root
+        
+        if (self.__current_device in self.__device_items):
+            strip_index = self.__device_items.index(self.__current_device)        
+            self.hilight_strip_item(strip_index)
+        
         self.set_title(device.get_name())
         if (root.mimetype == root.DIRECTORY):
             self.__load_folder(root, self.__GO_NEW)
         else:
             self.__load_file(root, self.__GO_NEW)
             
-        self.emit_event(msgs.UI_EV_DEVICE_SELECTED, device.get_device_id())
+        self.emit_message(msgs.UI_EV_DEVICE_SELECTED, device.get_device_id())
 
 
 
@@ -775,14 +828,15 @@ class GenericViewer(Viewer):
         """
 
         if (f.mimetype.endswith("-folder")):    
-            gobject.timeout_add(250, self.__load_folder, f, self.__GO_CHILD)
+            #gobject.timeout_add(250, self.__load_folder, f, self.__GO_CHILD)
+            self.__load_folder(f, self.__GO_CHILD)
                 
         else:
             self.__current_file = f
             self.set_title(f.name)
 
             if (not f.mimetype in mimetypes.get_image_types()):
-                self.emit_event(msgs.MEDIA_ACT_STOP)
+                self.emit_message(msgs.MEDIA_ACT_STOP)
 
             # request media widget
             media_widget = self.call_service(
@@ -823,9 +877,9 @@ class GenericViewer(Viewer):
 
             self.__hilight_current_file()
 
-            self.emit_event(msgs.UI_ACT_RENDER)
+            self.emit_message(msgs.UI_ACT_RENDER)
             self.__media_widget.load(f, direction)
-            self.emit_event(msgs.MEDIA_EV_LOADED, self, f)
+            self.emit_message(msgs.MEDIA_EV_LOADED, self, f)
 
             try:
                 idx = self.__playable_items.index(f)
@@ -845,50 +899,40 @@ class GenericViewer(Viewer):
         def on_child(f, path, entries, items_to_thumbnail):
             # abort if the user has changed the directory again
             if (self.__path_stack[-1][0] != path): return False
-
+            
             if (f):
                 self.__list.get_item(0).set_info("Loading (%d items)..." \
                                                  % len(self.__items))
+                self.__list.invalidate_image(0)
                 entries.append(f)
                 self.__add_file(f, items_to_thumbnail, -1)
 
             else:
                 self.__list.get_item(0).set_info("%d items" % len(self.__items))
+                self.__list.invalidate_image(0)
                 self.__random_items = self.__playable_items[:]
                 # finished loading items; now create thumbnails
                 self.__create_thumbnails(path, items_to_thumbnail)
+                self.__strip_needs_reload(self._VIEWMODE_PLAYER_NORMAL)
 
             now = time.time()
-            if (not f or len(entries) == 4 or \
-                  now > self.__last_list_render_time + 0.5):
+            #if (not f or len(entries) == 4 or \
+            if (not f or
+                  now > self.__last_list_render_time + 1.0):
                 self.__last_list_render_time = now
             #if (not f or len(entries) == 4 or len(entries) % 10 == 0):
-                self.__list.invalidate_buffer()
+                #self.__list.invalidate_image(len(entries) + 1) #buffer()
                 self.__list.render()
             
             return True
-
-        
-        def on_sibling(f, path, entries):
-            # abort if the user has changed the directory again
-            if (self.__path_stack[-2][0] != path): return False
-            
-            if (f and f.mimetype.endswith("-folder")):
-                entries.append(f)
-            else:
-                if (`entries` != `self.__sibling_folders`):
-                    self.__sibling_folders = entries
-                    self.__update_side_strip()
-
-            return True
-                    
-
+                   
 
         self.__close_subfolder()
                    
         # clear items
         self.__items = []
         self.__playable_items = []
+        self.__random_items = []
         self.__items_downloading_thumbnails.clear()
 
         # update path stack
@@ -903,33 +947,49 @@ class GenericViewer(Viewer):
         if (not reload_only):
             self.__path_stack.append([path, 0])
 
-        self.__list.clear_items()        
+
+        if (direction == self.__GO_PARENT):
+            # don't reload list
+            self.__list.clear_items()
+            self.__list.change_image_set(path.full_path)
+            self.__list.hilight(-1)
+            
+            self.__items = [ i.get_file() for i in self.__list.get_items()[1:] ]
+            self.__playable_items = [ i for i in self.__items 
+                                      if not i.mimetype.endswith("-folder") ]
+            self.__random_items = self.__playable_items[:]
+            self.__list.fx_slide_right()
+            
+        else:
+            # reload list           
+            self.__list.change_image_set(path.full_path)
+            self.__list.clear_items()
+
+            header = HeaderItem(path.name)
+            header.set_info("Retrieving...")
+            buttons = [(header.BUTTON_ENQUEUE, theme.mb_item_btn_enqueue)]
+            if (path.can_add_to_library):
+                buttons.append((header.BUTTON_ADD_TO_LIBRARY, theme.mb_item_btn_add))
+            header.set_buttons(*buttons)
+            self.__list.append_item(header)
+
+            self.__list.fx_slide_left()
+
+            gobject.timeout_add(0, path.get_children_async, on_child, path,
+                                [], [])
+
+        #end if
+            
         #if (direction == self.__GO_PARENT):
         #    self.__list.fx_slide_right()
         #elif (direction == self.__GO_CHILD):
         #    self.__list.fx_slide_left()
 
-        header = HeaderItem(path.name)
-        header.set_info("Retrieving...")
-        buttons = [(header.BUTTON_ENQUEUE, theme.mb_item_btn_enqueue)]
-        if (path.can_add_to_library):
-            buttons.append((header.BUTTON_ADD_TO_LIBRARY, theme.mb_item_btn_add))
-        header.set_buttons(*buttons)
-        self.__list.append_item(header)
-        self.__update_toolbar()
         #self.__list.render()
-        self.emit_event(msgs.UI_ACT_RENDER)
+        self.__update_toolbar()
+        self.emit_message(msgs.UI_ACT_RENDER)
+        import gc; gc.collect()
         
-        if (self.__view_mode == self._VIEWMODE_SPLIT_BROWSER):
-            if (len(self.__path_stack) > 1):
-                parent_path = self.__path_stack[-2][0]
-                gobject.timeout_add(0, parent_path.get_children_async,
-                                    on_sibling, parent_path, [])
-            else:
-                self.__sibling_folders = []
-                self.__update_side_strip()
-
-        gobject.timeout_add(0, path.get_children_async, on_child, path, [], [])
 
 
 
@@ -953,16 +1013,17 @@ class GenericViewer(Viewer):
 
             now = time.time()
             self.__subfolder_range = (insert_at, insert_at + len(entries))
-            if (not f or len(entries) == 4 or \
-                  now > self.__last_list_render_time + 0.5):
+            #if (not f or len(entries) == 4 or \
+            if (not f or
+                  now > self.__last_list_render_time + 1.0):
                 self.__last_list_render_time = now
                 self.__list.invalidate_buffer()
                 self.__list.render()
             
             return True
     
-        idx = self.__items.index(path)
-        self.__list.scroll_to_item(idx + 1, force_on_top = True)
+        #idx = self.__items.index(path)
+        #self.__list.scroll_to_item(idx + 1, force_on_top = True)
         self.__close_subfolder()
         idx = self.__items.index(path)
     
@@ -1001,19 +1062,23 @@ class GenericViewer(Viewer):
         Adds the given file item to the list.
         """
 
-        # lookup thumbnail        
-        icon, uptodate = self.call_service(msgs.MEDIASCANNER_SVC_GET_THUMBNAIL,
-                                           entry)
-
-        #tn = FileThumbnail(icon, entry)
-        #if (entry.mimetype != entry.DIRECTORY):
-        #    self.__thumbnails.append(tn)
-        
-        # determine available item buttons
         if (insert_at == -1):
-            item = MediaItem(entry, icon)
+            # look if there's a thumbnail available
+            if (entry.icon):
+                thumbnail = entry.icon
+            else:
+                thumbnail = self.call_service(
+                    msgs.MEDIASCANNER_SVC_LOOKUP_THUMBNAIL, entry) or None
+
+            item = MediaItem(entry, thumbnail)
+            # remember for thumbnailing if no thumbnail was found
+            if (not thumbnail and entry.mimetype != entry.DIRECTORY):
+                items_to_thumbnail.append((item, None, entry))
+
         else:
             item = SubItem(entry)
+
+        # determine available item buttons
         buttons = []
         
         if (entry.mimetype == "application/x-music-folder"):
@@ -1031,19 +1096,23 @@ class GenericViewer(Viewer):
         item.set_buttons(*buttons)
 
         # remember if thumbnail does not yet exist
-        if (insert_at == -1 and (not os.path.exists(icon) or not uptodate)):
-            if (entry.is_local or entry.thumbnail):
-                items_to_thumbnail.append((item, None, entry))
+        #if (insert_at == -1 and (not os.path.exists(icon))):
+        #    if (entry.is_local or entry.thumbnail):
+        #items_to_thumbnail.append((item, None, entry))
         #end if
         
-        self.__list.set_frozen(True)
+        if (self.may_render()):
+            self.__list.set_frozen(True)
+            
         if (insert_at == -1):
             self.__list.append_item(item)
             self.__items.append(entry)
         else:
             self.__list.insert_item(item, insert_at)
             self.__items.insert(insert_at, entry)
-        self.__list.set_frozen(False)
+            
+        if (self.may_render()):
+            self.__list.set_frozen(False)
         
         if (not entry.mimetype.endswith("-folder")):
             self.__playable_items.append(entry)
@@ -1069,7 +1138,7 @@ class GenericViewer(Viewer):
         logging.debug("reached EOF")
         self.__may_go_next = True
         
-        self.emit_event(msgs.MEDIA_EV_EOF)
+        self.emit_message(msgs.MEDIA_EV_EOF)
 
         if (self.__may_go_next):
             logging.debug("going to next item")
@@ -1081,7 +1150,7 @@ class GenericViewer(Viewer):
         Reacts on changing the sound volume.
         """
 
-        self.emit_event(msgs.MEDIA_EV_VOLUME_CHANGED, volume)
+        self.emit_message(msgs.MEDIA_EV_VOLUME_CHANGED, volume)
 
 
     def __go_previous(self):
@@ -1206,58 +1275,6 @@ class GenericViewer(Viewer):
 
         if (self.__current_file):
             self.__current_file.keep()
-            
-
-
-    def __on_download_thumbnail(self, d, a, t, f, data, path, items_to_thumbnail):
-        """
-        Callback for downloading a thumbnail image.
-        """
-
-        def on_size_available(loader, width, height):
-            if (width > 160 or height > 120):
-                factor = 1            
-                factor1 = 160 / float(width)
-                factor2 = 120 / float(height)
-                factor = min(factor1, factor2)
-                loader.set_size(int(width * factor), int(height * factor))
-            #end if
-            
-
-        data[0] += d
-        if (not d):
-            item, tn = self.__items_downloading_thumbnails.get(f, (None, None))
-            if (not item): return
-
-            data = data[0]
-            if (data):
-                try:
-                    loader = gtk.gdk.PixbufLoader()
-                    loader.connect("size-prepared", on_size_available)
-                    loader.write(data)
-                    loader.close()                                
-                    pbuf = loader.get_pixbuf()
-                except:
-                    gobject.idle_add(self.__create_thumbnails,
-                                     path, items_to_thumbnail)
-                    return
-                
-                thumbpath = self.call_service(msgs.MEDIASCANNER_SVC_SET_THUMBNAIL,
-                                              f, pbuf)
-                del pbuf
-                item.set_icon(thumbpath)
-                item.invalidate()
-                self.__list.invalidate_buffer()
-                self.__list.render()
-                
-                #tn.set_thumbnail(thumbpath)
-                #tn.invalidate()
-                if (self.__view_mode == self._VIEWMODE_PLAYER_NORMAL):
-                    self.emit_event(msgs.CORE_ACT_RENDER_ITEMS)
-            #end if
-            
-            gobject.idle_add(self.__create_thumbnails, path, items_to_thumbnail)
-        #end if
 
 
     def __create_thumbnails(self, path, items_to_thumbnail):
@@ -1265,20 +1282,23 @@ class GenericViewer(Viewer):
         Creates thumbnails for the given items.
         """
 
-        def on_created(thumbpath, item, tn):
-            item.set_icon(thumbpath)
-            item.invalidate()
-            self.__list.invalidate_buffer()
-            self.__list.render()
+        def on_loaded(thumbpath, item, tn):
+            if (item):
+                item.set_icon(thumbpath)
+                item.invalidate()
+                self.__list.invalidate_buffer()
+                self.__list.render()
             
-            #tn.set_thumbnail(thumbpath)
-            #tn.invalidate()
+            if (tn):
+                tn.set_thumbnail(thumbpath)
+                tn.invalidate()
             
             if (self.__view_mode == self._VIEWMODE_PLAYER_NORMAL):
-                self.emit_event(msgs.CORE_ACT_RENDER_ITEMS)
+                self.emit_message(msgs.CORE_ACT_RENDER_ITEMS)
 
             # proceed to next thumbnail
-            gobject.idle_add(self.__create_thumbnails, path, items_to_thumbnail)
+            gobject.idle_add(self.__create_thumbnails, path,
+                             items_to_thumbnail)
             
 
         # abort if the user has changed the directory again
@@ -1288,7 +1308,7 @@ class GenericViewer(Viewer):
     
         if (items_to_thumbnail):
             # hmm, may we want to reorder a bit?
-            if (len(items_to_thumbnail) % 10 == 0 and\
+            if (len(items_to_thumbnail) % 10 == 0 and \
                   len(items_to_thumbnail) > 5):
                 idx_in_list = self.__list.get_index_at(0)
                 item_in_view = self.__items[idx_in_list - 1]
@@ -1305,28 +1325,12 @@ class GenericViewer(Viewer):
             #end if            
         
             item, tn, f = items_to_thumbnail.pop(0)
-            if (f.thumbnail):
-                # a thumbnail URI is specified
-                self.__items_downloading_thumbnails[f] = (item, tn)
-                print "GET FROM URI", f.thumbnail
-                if (f.thumbnail.startswith("/")):
-                    try:
-                        self.__on_download_thumbnail("", 0, 0, f,
-                                                     [open(f.thumbnail).read()],
-                                                     path, items_to_thumbnail)
-                    except:
-                        pass
-                else:
-                    Downloader(f.thumbnail, self.__on_download_thumbnail, f,
-                               [""], path, items_to_thumbnail)
-            else:
-                # create new thumbnail
-                self.call_service(msgs.MEDIASCANNER_SVC_CREATE_THUMBNAIL, f,
-                                  on_created, item, tn)
 
-            #end if
-    
+            # load thumbnail
+            self.call_service(msgs.MEDIASCANNER_SVC_LOAD_THUMBNAIL, f,
+                              on_loaded, item, tn)
         #end if
+            
 
 
     def __search(self, key):
@@ -1346,13 +1350,16 @@ class GenericViewer(Viewer):
     def show(self):
         
         def f():
-            if (self.may_render() and not self.__current_device):
-                self.select_strip_item(0)
+            if (self.may_render()):
+                if (not self.__current_device):
+                    self.select_strip_item(0)
                 return False
             else:
                 return True
     
         Viewer.show(self)
+        self.change_strip((self, self.__view_mode))
+        self.__update_side_strip()
         
         if (not self.__current_device):
             self.__list.clear_items()
@@ -1364,7 +1371,7 @@ class GenericViewer(Viewer):
     def hide(self):
     
         if (self.__lib_is_dirty):
-            self.emit_event(msgs.CORE_ACT_SCAN_MEDIA, False)
+            self.emit_message(msgs.CORE_ACT_SCAN_MEDIA, False)
             self.__lib_is_dirty = False
         Viewer.hide(self)
 
