@@ -14,7 +14,8 @@ import gc
 
 
 # predefined zoom levels
-_ZOOM_LEVELS = [12, 25, 50, 100, 200, 400, 800, 1600, 3200]
+_ZOOM_LEVELS = [12, 18, 25, 36, 50, 75, 100, 150, 200, 300, 400, 600,
+                800, 1200, 1600, 2400, 3200]
 
 # read this many bytes at once
 _CHUNK_SIZE = 50000
@@ -47,7 +48,8 @@ class Image(MultiTouchWidget, Observable):
         self.__bg_color = gtk.gdk.color_parse("#000000")
         
         # offscreen drawing pixmap
-        self.__offscreen = Pixmap(None, 800, 480)
+        self.__offscreen = Pixmap(None, gtk.gdk.screen_width(),
+                                        gtk.gdk.screen_height())
 
         # original size of the image
         self.__original_size = (0, 0)
@@ -128,7 +130,9 @@ class Image(MultiTouchWidget, Observable):
 
         # create a client-side pixmap for rendering
         self.__buffer = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB,
-                                       True, 8, 800, 480)
+                                       True, 8,
+                                       gtk.gdk.screen_width(),
+                                       gtk.gdk.screen_height())
 
 
     def render_this(self):
@@ -352,7 +356,7 @@ class Image(MultiTouchWidget, Observable):
         from_value = self.__zoom_value
         to_value = self.__zoom_levels[level] / 100.0
         
-        self.animate(50, f, [from_value, to_value])
+        self.animate(25, f, [from_value, to_value])
         self.__zoom_level = level
         
       
@@ -676,7 +680,9 @@ class Image(MultiTouchWidget, Observable):
             
 
         # automatically rotate images in portrait format
-        if (pbuf.get_width() < pbuf.get_height()):
+        w, h = self.get_size()
+        if (w > h and pbuf.get_width() < pbuf.get_height() or
+            w < h and pbuf.get_width() > pbuf.get_height()):
             try:
                 # rotating is only supported by pygtk >= 2.10
                 pixbuf = pbuf.rotate_simple(
