@@ -46,12 +46,18 @@ class PlaylistViewer(Viewer):
         # list of thumbnails representing the playlists
         self.__pl_thumbnails = []
     
-        self.__random_items = []
+        self.__random_items = []        
     
         self.__current_file = None
         self.__media_widget = None
         self.__view_mode = _VIEWMODE_NONE
     
+        # table: owner -> needs_reload
+        self.__needs_reload = {}
+        
+        self.__needs_playlist_reload = False
+
+    
         Viewer.__init__(self)
         
         self.__playlist = TrackList(with_drag_sort = True)
@@ -90,13 +96,13 @@ class PlaylistViewer(Viewer):
         ]
         self.set_toolbar(self.__playlist_tbset)
 
-        #self.__set_view_mode(_VIEWMODE_PLAYLIST)
 
         self.__side_tabs.add_tab(None, "Playlist",
                                  self.__set_view_mode, _VIEWMODE_PLAYLIST)
         self.__side_tabs.add_tab(None, "Player",
                                  self.__set_view_mode, _VIEWMODE_PLAYER)
         gobject.idle_add(self.__load_playlists)
+        gobject.timeout_add(0, self.__set_view_mode, _VIEWMODE_PLAYLIST)
 
 
     def render_this(self):
@@ -129,13 +135,16 @@ class PlaylistViewer(Viewer):
 
 
     def __get_playlist_thumbnail_pieces(self, pl):
+        """
+        Returns the thumbnails that will be used for rendering the icon for the
+        given playlist.
+        """
     
         pieces = []
         for f in pl.get_files():
             if (not f): continue
-            thumb, uptodate = self.call_service(
-                                        msgs.MEDIASCANNER_SVC_GET_THUMBNAIL, f)
-            if (not (thumb, f.mimetype) in pieces and os.path.exists(thumb)):
+            thumb = self.call_service(msgs.MEDIASCANNER_SVC_LOOKUP_THUMBNAIL, f) or None
+            if (thumb and not (thumb, f.mimetype) in pieces):
                 pieces.append((thumb, f.mimetype))
             if (len(pieces) == 4):
                 break
@@ -145,12 +154,68 @@ class PlaylistViewer(Viewer):
 
 
     def __update_playlist_thumbnail(self):
+        """
+        Updates the thumbnail of the current playlist.
+        """
     
         pl_tn = self.__pl_thumbnails[self.__current_list]
         pl = self.__lists[self.__current_list]
         pl_tn.set_thumbnails(self.__get_playlist_thumbnail_pieces(pl))
+        self.__strip_needs_reload(_VIEWMODE_PLAYLIST)
+
+
+    def __strip_needs_reload(self, view_mode):
+        """
+        Marks the current side strip to need a reload.
+        """
+        
+        strip_owner = (self, view_mode)
+        self.__needs_reload[strip_owner] = True
+        
+
+    def __update_side_strip(self):
+        """
+        Updates the side strip depending on the current view mode.
+        """
+        
+        strip_items = []
+        strip_index = -1
+        
+        if (not self.is_active()): return
+        
+        strip_owner = (self, self.__view_mode)
+        needs_reload = self.__needs_reload.get(strip_owner, True)
+        items_to_thumbnail = []
+        
+        if (not needs_reload):            
+            return
+        else:
+            self.__needs_reload[strip_owner] = False
+
+        pl = self.__lists[self.__current_list]
+        pl_tn = self.__pl_thumbnails[self.__current_list]
+
+        # show devices
         if (self.__view_mode == _VIEWMODE_PLAYLIST):
-            self.set_strip(self.__pl_thumbnails)
+            strip_items = self.__pl_thumbnails
+            strip_index = self.__current_list
+
+        # show playable items
+        elif (self.__view_mode == _VIEWMODE_PLAYER):
+            for tn in pl.get_thumbnails():
+                #items_to_thumbnail.append((None, tn, entry))
+                strip_items.append(tn)
+            #end for
+            
+            if (self.__current_file in pl.get_files()):
+                strip_index = pl.get_files().index(self.__current_file)
+
+        self.set_strip(strip_items)
+        if (strip_index >= 0):
+            self.hilight_strip_item(strip_index)
+
+        #if (self.__path_stack):
+        #    self.__create_thumbnails(self.__path_stack[-1][0], items_to_thumbnail)
 
 
     def __update_input_context(self):
@@ -262,9 +327,11 @@ class PlaylistViewer(Viewer):
             pl_tn = PlaylistThumbnail(pl)
             self.__pl_thumbnails.append(pl_tn)
         
-            self.set_strip(self.__pl_thumbnails)
-            self.select_strip_item(len(self.__lists) - 1)
+            #self.set_strip(self.__pl_thumbnails)
+            #self.select_strip_item(len(self.__lists) - 1)
         #end if
+        self.__strip_needs_reload(_VIEWMODE_PLAYLIST)
+        self.__update_side_strip()
         
         
     def __delete_playlist(self):
@@ -281,7 +348,9 @@ class PlaylistViewer(Viewer):
             pl.delete_playlist()            
             self.__load_playlists()
             self.__update_playlist_thumbnail()
-            self.select_strip_item(0)
+            #self.select_strip_item(0)
+            
+        self.__update_side_strip()
 
 
     def __on_toggle_fullscreen(self):
@@ -310,6 +379,7 @@ class PlaylistViewer(Viewer):
             self.hilight_strip_item(self.__current_list)
 
             self.emit_event(msgs.UI_ACT_THAW)
+            self.emit_event(msgs.UI_ACT_RENDER)
 
         elif (mode == _VIEWMODE_PLAYER):
             self.emit_event(msgs.UI_ACT_FREEZE)
@@ -325,7 +395,6 @@ class PlaylistViewer(Viewer):
 
             self.emit_event(msgs.UI_ACT_THAW)
             self.emit_event(msgs.UI_ACT_RENDER)
-                
 
         elif (mode == _VIEWMODE_PLAYER_FULLSCREEN):
             self.emit_event(msgs.UI_ACT_VIEW_MODE, viewmodes.FULLSCREEN)
@@ -349,6 +418,7 @@ class PlaylistViewer(Viewer):
             self.__remove_item(pl, idx)
             pl.save()
             self.__update_playlist_thumbnail()
+            self.__update_side_strip()
                 
         elif (button == item.BUTTON_REMOVE_PRECEEDING):
             self.__playlist.set_frozen(True)
@@ -357,6 +427,7 @@ class PlaylistViewer(Viewer):
             pl.save()
             self.__playlist.set_frozen(False)
             self.__update_playlist_thumbnail()
+            self.__update_side_strip()
 
         elif (button == item.BUTTON_REMOVE_FOLLOWING):
             self.__playlist.set_frozen(True)
@@ -365,6 +436,8 @@ class PlaylistViewer(Viewer):
             pl.save()
             self.__playlist.set_frozen(False)
             self.__update_playlist_thumbnail()
+            self.__update_side_strip()
+
 
 
     def __on_swap(self, idx1, idx2):
@@ -574,8 +647,7 @@ class PlaylistViewer(Viewer):
                 return
             #end if
             
-            thumb, uptodate = self.call_service(
-                                        msgs.MEDIASCANNER_SVC_GET_THUMBNAIL, f)
+            thumb = self.call_service(msgs.MEDIASCANNER_SVC_LOOKUP_THUMBNAIL, f)
         else:
             thumb = None
 
@@ -586,7 +658,7 @@ class PlaylistViewer(Viewer):
 
         pl.append(plitem, tn, f)
         self.__random_items.append(f)
-        
+        self.__strip_needs_reload(_VIEWMODE_PLAYER)
 
 
     def __remove_item(self, pl, idx):
@@ -617,6 +689,7 @@ class PlaylistViewer(Viewer):
             self.__random_items.append(f)
         self.__playlist.invalidate_buffer()
         self.__playlist.render()
+        self.__strip_needs_reload(_VIEWMODE_PLAYER)
         
         
     def handle_message(self, msg, *args):
@@ -651,7 +724,8 @@ class PlaylistViewer(Viewer):
                 self.drop_event()
 
         elif (msg == msgs.CORE_EV_DEVICE_ADDED):
-            self.__load_playlists()
+            self.__needs_playlist_reload = True
+            #self.__load_playlists()
 
         elif (msg == msgs.MEDIA_ACT_STOP):
             if (self.__media_widget):
@@ -711,7 +785,7 @@ class PlaylistViewer(Viewer):
         # watch FULLSCREEN hw key
         if (msg == msgs.HWKEY_EV_FULLSCREEN):
             if (self.__view_mode in \
-                (_VIEWMODE_PLAYER, _VIEWMODE_PLAYER_FULLSCREEN)):
+                (_VIEWMODE_PLAYLIST, _VIEWMODE_PLAYER, _VIEWMODE_PLAYER_FULLSCREEN)):
                 self.__on_toggle_fullscreen()
                 
         # watch INCREMENT hw key
@@ -736,3 +810,16 @@ class PlaylistViewer(Viewer):
                 break
             idx += 1
         #end for
+        
+        
+    def show(self):
+    
+        Viewer.show(self)
+        
+        if (self.__needs_playlist_reload):
+            self.__load_playlists()
+            self.__needs_playlist_reload = False
+        
+        self.change_strip((self, self.__view_mode))
+        self.__update_side_strip()
+
