@@ -177,7 +177,6 @@ class AbstractBackend(Observable):
 
     def __resume_if_necessary(self):
     
-        print "TRY RESUME"
         if (self.__suspension_point):        
             uri, pos = self.__suspension_point
             self.__suspension_point = None
@@ -234,8 +233,12 @@ class AbstractBackend(Observable):
                     self.update_observer(self.OBS_POSITION, self.__context_id,
                                          pos, total)
 
+            if (total != 0 and total - pos < 1):
+                delay = 200
+            else:
+                delay = 600
             self.__position_handler = \
-                  gobject.timeout_add(300, self.__update_position,
+                  gobject.timeout_add(delay, self.__update_position,
                                       beginpos, timestamp)
 
             # detect EOF
@@ -303,7 +306,7 @@ class AbstractBackend(Observable):
         self._load(uri)
         self._set_volume(self.__volume)
         
-        self.play()
+        gobject.timeout_add(0, self.play)
 
         if (ctx_id != -1):
             self.__context_id = ctx_id
@@ -369,7 +372,6 @@ class AbstractBackend(Observable):
         
         self.__resume_if_necessary()
 
-        print "PLAY"
         if (not self.__playing):
             self.__position = (0, 0)
             self._play()
@@ -402,7 +404,6 @@ class AbstractBackend(Observable):
         Stops playback.
         """
 
-        print "STOP"
         if (self.__playing):
             self._stop()
             self.update_observer(self.OBS_STOPPED, self.__context_id)
@@ -426,6 +427,8 @@ class AbstractBackend(Observable):
         @param volume: volume as a value between 0 and 100
         """
 
+        self._ensure_backend()
+        self.__volume = volume
         self._set_volume(volume)    
         
         
