@@ -71,6 +71,9 @@ class File(object):
         self.thumbnail = ""
         """may contain the URI of a thumbnail image"""
         
+        self.icon = ""
+        """may contain the path of an icon image"""
+        
         self.mimetype = self.FILE
         """MIME type of the file"""
                
@@ -168,10 +171,15 @@ class File(object):
                 return None
             
             longest = ""
+            blacklist = ["/sys", "/proc", "/dev"]
             for line in mounts:
                 parts = line.split()
                 mountpoint = parts[1]
-                if (uri.startswith(mountpoint)):
+                
+                if (mountpoint in blacklist):
+                    continue
+                
+                elif (uri.startswith(mountpoint)):
                     if (len(mountpoint) > len(longest)):
                         longest = mountpoint
             #end for
