@@ -1,7 +1,14 @@
+"""
+B{Used internally}
+"""
+
 import gtk
 
 
 class Pixbuf(gtk.gdk.Pixbuf):
+    """
+    Class for pixbuf theme elements.
+    """
 
     def __init__(self, path):
     
@@ -26,6 +33,11 @@ class Pixbuf(gtk.gdk.Pixbuf):
         self.__needs_reload = True
 
 
+    def get_path(self):
+    
+        return self.__path
+
+
     def reload(self):
     
         if (self.__needs_reload):
