@@ -5,13 +5,20 @@ A scrollable strip of images.
 from Widget import Widget
 from Pixmap import Pixmap, TEMPORARY_PIXMAP
 from SharedPixmap import SharedPixmap
+from utils import logging
 
 import gtk
 import gobject
-import threading
 
 
-_BPP = gtk.gdk.get_default_root_window().get_depth()
+class _ImageSet(object):
+    
+    def __init__(self):
+        
+        self.images = []
+        self.totalsize = 0
+        self.offset = 0
+        self.hilighted = -1
 
 
 class ImageStrip(Widget):
@@ -52,11 +59,18 @@ class ImageStrip(Widget):
         # shared pixmap for storing the items
         self.__shared_pmap = None
     
+        # sets of images; table: owner -> set
+        self.__image_sets = {}
+        self.__current_image_set = None
+    
+        # indices of images that have been invalidated
+        self.__invalidated_images = []
+    
         Widget.__init__(self)
 
         self.__images = []
         self.__bg = None
-        self.__bg_color = None
+        self.__bg_color = "#ffffff"
         self.__offset = 0
         self.__canvas = None
         self.__cmap = None
@@ -71,17 +85,18 @@ class ImageStrip(Widget):
         self.__wrap_around = True
         
         self.__buffer = Pixmap(None, 100, 100)
-        self.__buffer_dirty = False
+        #self.__buffer_dirty = False
  
         self.set_size(100, 100)
  
  
     def _reload(self):
         """
-        Reload graphics when theme has changed.
+        Reloads graphics when theme has changed.
         """
 
         self.invalidate_buffer()
+        self.__clear_buffer()
 
         if (self.__scrollbar_pbuf):
             self.set_scrollbar(self.__scrollbar_pbuf)
@@ -101,7 +116,27 @@ class ImageStrip(Widget):
         contents of an item you have to invoke this method manually.
         """
 
-        self.__buffer_dirty = True
+        #self.__buffer_dirty = True
+        self.__invalidated_images = list(range(len(self.__images)))
+        
+        
+    def __clear_buffer(self):
+    
+        w, h = self.get_size()
+        self.__buffer.fill_area(0, 0, w, h, self.__bg_color)
+        
+        
+    def invalidate_image(self, idx):
+        """
+        Invalidates a single image. Rendering the list will only render the
+        invalidated images.
+        
+        @param idx: index of image
+        """
+        
+        #print "invalidating", idx
+        if (not idx in self.__invalidated_images):
+            self.__invalidated_images.append(idx)
 
  
     def set_size(self, w, h):
@@ -126,10 +161,9 @@ class ImageStrip(Widget):
 
     def set_bg_color(self, color):
     
-        w, h = self.get_size()
-        self.invalidate_buffer()
         self.__bg_color = color
-        self.__buffer.fill_area(0, 0, w, h, self.__bg_color)
+        self.invalidate_buffer()
+        self.__clear_buffer()
         
         
     def set_background(self, bg):
@@ -137,8 +171,9 @@ class ImageStrip(Widget):
         Sets the background image from the given pixbuf or filename.
         """
     
-        self.invalidate_buffer()
         self.__bg = bg
+        self.invalidate_buffer()
+        self.__clear_buffer()
         
         
     def set_caps(self, top, bottom):
@@ -153,6 +188,7 @@ class ImageStrip(Widget):
     
         w, h = self.get_size()
         self.invalidate_buffer()
+        self.__clear_buffer()
         self.__caps = (top, bottom)
 
         if (top):
@@ -191,6 +227,7 @@ class ImageStrip(Widget):
         """
 
         self.invalidate_buffer()
+        self.__clear_buffer()
     
         arrow_up = None
         arrow_down = None
@@ -237,6 +274,7 @@ class ImageStrip(Widget):
         """
     
         self.invalidate_buffer()
+        self.__clear_buffer()
     
         self.__scrollbar_pbuf = pbuf
         if (not pbuf):
@@ -259,12 +297,45 @@ class ImageStrip(Widget):
         @param pos: position where to display the item
         """
 
-        self.invalidate_buffer()
+        if (idx >= 0):
+            self.invalidate_image(idx)
+        else:
+            self.invalidate_image(self.__floating_index)
+        #self.invalidate_buffer()
+        #self.__clear_buffer()
         
         self.__floating_index = idx
         self.__floating_position = pos
 
 
+    def change_image_set(self, owner):
+        """
+        Changes the current image set.
+        
+        @param owner: owner of the set to select
+        """
+        
+        if (self.__current_image_set):
+            self.__current_image_set.images = self.__images
+            self.__current_image_set.offset = self.__offset
+            self.__current_image_set.totalsize = self.__totalsize
+            self.__current_image_set.hilighted = self.__hilighted_image
+        
+        imgset = self.__image_sets.get(owner, _ImageSet())
+        self.__image_sets[owner] = imgset
+        
+        self.__images = imgset.images
+        self.__offset = imgset.offset
+        self.__totalsize = imgset.totalsize
+        self.__hilighted_image = imgset.hilighted
+        self.__current_image_set = imgset
+
+        for image in self.__images:
+            image.invalidate()
+        self.invalidate_buffer()
+        self.__clear_buffer()
+
+
     def get_image(self, idx):
         """
         Returns the item with the given index.
@@ -297,10 +368,13 @@ class ImageStrip(Widget):
         while (self.__images):
             img = self.__images.pop()
             del img  
-        self.__images = []
+
         self.__hilighted_image = -1
         
         import gc; gc.collect()
+        
+        self.__invalidated_images = []
+        #self.invalidate_buffer()
 
         # append new images
         for img in images:
@@ -308,6 +382,7 @@ class ImageStrip(Widget):
         #endfor
 
         #self.invalidate_buffer()
+        self.__clear_buffer()
         self.__offset = 0
         
         
@@ -326,7 +401,13 @@ class ImageStrip(Widget):
         img.set_size(w, img_h)
         img.set_hilighted(False)
 
-        self.invalidate_buffer()
+        #self.invalidate_buffer()
+        
+        is_scrollable = self.__is_scrollable()
+        
+        idx = len(self.__images)
+        self.invalidate_image(idx)
+        
         if (not self.__shared_pmap):
             self.__shared_pmap = SharedPixmap(w, img_h)
 
@@ -339,6 +420,9 @@ class ImageStrip(Widget):
 
         #if (len(self.__images) < 30): self.__shared_pmap.prepare(img)
         
+        if (is_scrollable != self.__is_scrollable):
+            self.invalidate_buffer()
+        
         return len(self.__images) - 1
 
 
@@ -357,6 +441,7 @@ class ImageStrip(Widget):
         img.set_size(w, img_h)
 
         self.invalidate_buffer()
+        self.__clear_buffer()
         if (not self.__shared_pmap):
             self.__shared_pmap = SharedPixmap(w, img_h)
 
@@ -376,7 +461,8 @@ class ImageStrip(Widget):
        @param image: StripItem object with which to replace
        """
 
-       self.invalidate_buffer()
+       #self.invalidate_buffer()
+       self.invalidate_image(idx)
        img = self.__images[idx]
        self.__images[idx] = image
        del img
@@ -395,6 +481,7 @@ class ImageStrip(Widget):
         """
     
         self.invalidate_buffer()
+        self.__clear_buffer()
         w, h = self.get_size()
         del self.__images[idx]
         self.__totalsize = (self.__itemsize + self.__gapsize) * len(self.__images)        
@@ -408,8 +495,9 @@ class ImageStrip(Widget):
         elif (idx < self.__hilighted_image):
             self.__hilighted_image -= 1
 
-        if (self.__totalsize < h):
-            self.__offset = 0
+        #if (self.__totalsize < self.__offset + h):
+        #    self.__offset = max(self.__totalsize - h, 0)
+        self.__fix_offset()
 
         #self.render()
 
@@ -421,8 +509,10 @@ class ImageStrip(Widget):
         a time.
         """
 
-        self.invalidate_buffer()
+        #self.invalidate_buffer()
+        
         if (self.__hilighted_image >= 0):
+            self.invalidate_image(self.__hilighted_image)
             try:
                 item = self.get_image(self.__hilighted_image)
                 item.set_hilighted(False)
@@ -431,6 +521,7 @@ class ImageStrip(Widget):
 
         if (idx >= 0 and idx < len(self.__images)):
             item = self.get_image(idx)
+            self.invalidate_image(idx)
             item.set_hilighted(True)
             self.__hilighted_image = idx
             self.render()
@@ -448,7 +539,8 @@ class ImageStrip(Widget):
         
         if (not value):
             w, h = self.get_size()
-            self.__offset = max(0, min(self.__offset, self.__totalsize - h))
+            self.__fix_offset()
+            #self.__offset = max(0, min(self.__offset, self.__totalsize - h))
 
     def __is_wrap_around(self):
     
@@ -486,21 +578,21 @@ class ImageStrip(Widget):
             cw, ch = self.__cap_top_size
             y -= ch
         
-        if (not self.__images or y > self.__totalsize):
-            return -1, 0
-        else:
-            blocksize = self.__itemsize + self.__gapsize
-            pos = self.__offset + y            
-            index = (pos / blocksize)
-            inside_y = (float(pos) / blocksize) - index
+        if (not self.__images):
+            return (-1, 0)
             
-            if (index < 0 or index >= len(self.__images)):
-                if (self.__is_wrap_around()):
-                    index %= len(self.__images)
-                else:
-                    index = -1
+        blocksize = self.__itemsize + self.__gapsize
+        pos = self.__offset + y
+        index = (pos / blocksize)
+        inside_y = (float(pos) / blocksize) - index
         
-            return index, inside_y
+        if (self.__is_wrap_around()):
+            index %= len(self.__images)
+        elif (index >= len(self.__images)):
+            print "out of range", index, len(self.__images)
+            return (-1, 0)
+
+        return (index, inside_y)
             
             
     def swap(self, idx1, idx2):
@@ -511,7 +603,10 @@ class ImageStrip(Widget):
         @param idx2: place 2
         """
         
-        self.invalidate_buffer()
+        #self.invalidate_buffer()
+        self.invalidate_image(idx1)
+        self.invalidate_image(idx2)
+        
         temp = self.__images[idx1]
         self.__images[idx1] = self.__images[idx2]
         self.__images[idx2] = temp
@@ -556,6 +651,33 @@ class ImageStrip(Widget):
         w, h = self.get_size()
         offset = min(offset, max(0, self.__totalsize - h))
         self.__offset = offset
+        self.__fix_offset()
+        
+        
+    def __fix_offset(self):
+        """
+        Fixes the offset value to be in a valid range.
+        """
+        
+        w, h = self.get_size()
+        
+        if (not self.__is_scrollable()):
+            self.__offset = 0
+        
+        elif (self.__is_wrap_around()):
+            while (self.__offset < 0):
+                self.__offset += self.__totalsize
+            while (self.__offset > self.__totalsize):
+                self.__offset -= self.__totalsize
+
+        else:
+            if (self.__offset < 0):
+                self.__offset = 0
+            elif (self.__offset > self.__totalsize - h):
+                self.__offset = self.__totalsize - h
+
+        #end if
+
         
         
     def __render_arrows(self):
@@ -705,8 +827,9 @@ class ImageStrip(Widget):
             self.__buffer.draw_pixbuf(self.__cap_bottom, 0, h - ch)
 
         screen.copy_pixmap(self.__buffer, 0, offset, x, y + offset, w, height)
+        #print "COPY BUFFER"
         self.__buffer.copy_pixmap(TEMPORARY_PIXMAP, 0, 0, 0, 0, w, h)
-        self.__buffer_dirty = False
+        #self.__buffer_dirty = False
 
         
     def render_this(self):
@@ -714,13 +837,16 @@ class ImageStrip(Widget):
         w, h = self.get_size()
         screen = self.get_screen()
 
-        if (self.__buffer_dirty):
+        if (not self.may_render()): return
+
+        #if (self.__buffer_dirty):
+        if (self.__invalidated_images):
             self.render_full()
             self.__render_buffered(screen, 0, h)
             
         else:
             # nothing changed; simply render the buffer again
-            print "restoring from buffer"
+            logging.debug("restoring list from buffer")
             self.__render_buffered(screen, 0, h)
         
         
@@ -744,39 +870,86 @@ class ImageStrip(Widget):
    
         if (not self.may_render()): return
 
-        
+        #print "render", render_offset, render_height, "{"
         blocksize = self.__itemsize + self.__gapsize
         render_to = render_offset + render_height
-
-        x, y = 0, 0 #self.get_screen_pos()
         w, h = self.get_size()
-        screen = self.__buffer
-        if (not self.__wrap_around and self.__totalsize > h and 
-              self.__offset > self.__totalsize - h):
-            self.__offset = self.__totalsize - h
+
+        #self.__buffer.fill_area(0, render_offset, w, render_height,
+        #                        self.__bg_color)
 
         if (not self.__is_scrollable()):
             cw, ch = self.__cap_top_size
-            screen.fill_area(x, y, w, ch, self.__bg_color)
-            y += ch
-            h -= ch
+            render_offset += ch
 
-        # render items
-        while (self.__images and render_offset < render_to):            
-            idx = ((self.__offset + render_offset) / blocksize)
+        idx1 = self.get_index_at(render_offset)
+        if (idx1 == -1):
+            # nothing to render
+            return
+        
+        y = render_offset
+        idx = idx1
+        len_img = len(self.__images)
+        is_wrap_around = self.__is_wrap_around()
+        while (y < render_to + blocksize):
+            m_idx = idx % len_img
+            if (m_idx >= len_img and not is_wrap_around):
+                break
 
-            # wrap around is not available when there are too few items
-            if (self.__is_wrap_around()):
-                idx %= len(self.__images)
-            elif (idx >= len(self.__images) or idx < 0):
-                break                            
+            if (m_idx in self.__invalidated_images):
+                #print "  ", m_idx, self.is_frozen(), self.is_visible()
+                self.__render_item(m_idx)
+                self.__invalidated_images.remove(m_idx)
+                
+            y += blocksize
+            idx += 1
+        #end while
+        #print "}"
 
-            img_offset = (self.__offset + render_offset) % blocksize
 
-            # compute the remaining visible part of the item            
-            remain = min(self.__itemsize - img_offset, render_to - render_offset)
-            if (remain > 0):
-                # prepare item
+
+    def __render_item(self, idx):
+        """
+        Renders the given single item onto the buffer.
+        """
+
+        w, h = self.get_size()
+        blocksize = self.__itemsize + self.__gapsize
+        item_y = idx * blocksize - self.__offset
+        item_height = self.__itemsize
+
+        if (not self.__is_scrollable()):
+            cw, ch = self.__cap_top_size
+            item_y = idx * blocksize + ch
+
+        item = None
+        offx = 0
+        pw = 0
+
+        while (item_y < h):
+            # is the item on screen?
+            if (item_y < 0 - item_height):
+                item_y += self.__totalsize
+                continue
+        
+       
+            # determine visible part of item
+            if (item_y < 0):
+                item_top = 0
+                item_offset = abs(item_y)
+            else:
+                item_top = item_y
+                item_offset = 0
+                
+            if (item_y + item_height > h):
+                item_remain = h - item_y - item_offset
+            else:
+                item_remain = item_height - item_offset
+        
+            #print item_y, "->", item_height, "(", item_top, item_offset, item_remain, ")"
+        
+            # render item
+            if (not item):
                 item = self.__images[idx]
                 self.__shared_pmap.prepare(item)
                 pw, ph = item.get_size()
@@ -786,76 +959,45 @@ class ImageStrip(Widget):
                     offx = (w - pw) / 2
                 else:
                     offx = 0
-
-                if (idx == self.__floating_index):
-                    # leave a gap where the floating item would have been
-                    screen.fill_area(x + offx, y + render_offset, pw, remain,
-                                     self.__bg_color)
-
-                else:
-                    # render item normally
-                    screen.copy_pixmap(self.__shared_pmap, 0, img_offset,
-                                       x + offx, y + render_offset,
-                                       pw, remain)
-
-                # fill the empty space at the sides if the item was centered
-                if (offx > 0 and self.__bg_color):
-                    screen.fill_area(x, y + render_offset,
-                                     offx, remain,
-                                     self.__bg_color)
-                    screen.fill_area(x + offx + pw, y + render_offset,
-                                     offx, remain,
-                                     self.__bg_color)
-            #end if
-            
-            render_offset += (remain + self.__gapsize)
-        #end while
-        
-        # render the space not covered by items
-        if (self.__bg_color):
-
-            # fill the space beneath the items, if any
-            if (render_offset < render_to):
-                screen.fill_area(x, y + render_offset, w, render_to - render_offset,
-                                 self.__bg_color)
-                #screen.draw_subpixbuf(self.__bg,
-                #                      0, render_offset, x, y + render_offset,
-                #                      w, render_to - render_offset)
             #end if
 
+            if (idx == self.__floating_index):
+                # leave a gap where the floating item would have been
+                self.__buffer.fill_area(offx, item_top,
+                                        pw, item_remain,
+                                        self.__bg_color)
+
+            else:
+                # render item normally
+                self.__buffer.copy_pixmap(self.__shared_pmap,
+                                          0, item_offset,
+                                          offx, item_top,
+                                          pw, item_remain)
+
+            # fill the empty space at the sides if the item was centered
+            if (offx > 0 and self.__bg_color):
+                self.__buffer.fill_area(0, item_top,
+                                        offx, item_remain,
+                                       self.__bg_color)
+                self.__buffer.fill_area(offx + pw, item_top,
+                                        offx, item_remain,
+                                        self.__bg_color)
+
+            # fill gap between items
             if (self.__gapsize > 0):
-                # compute position of first gap
-                gap_offset = 0 - (self.__offset % blocksize) - self.__gapsize
-                
-                # render gap by gap
-                while (gap_offset < h):
-                    if (gap_offset + self.__gapsize < 0):
-                        # gap is off screen
-                        gap_offset += self.__gapsize + self.__itemsize
-                        continue
-                    elif (gap_offset < 0):
-                        # gap is partially on screen
-                        render_offset = 0
-                        remain = self.__gapsize - abs(gap_offset)
-                    else:
-                        # gap is on screen
-                        render_offset = gap_offset
-                        remain = min(self.__gapsize, h - render_offset)
-                        
-                    # render gap if visible
-                    if (remain > 0):
-                        screen.fill_area(x, y + render_offset, w, remain,
-                                         self.__bg_color)
-                        #screen.draw_subpixbuf(self.__bg,
-                        #                   0, render_offset, x, y + render_offset,
-                        #                   min(w, self.__bg.get_width()),
-                        #                   min(remain, self.__bg.get_height()))
-                    #end if
+                self.__buffer.fill_area(0, item_top - self.__gapsize,
+                                        pw, self.__gapsize,
+                                        self.__bg_color)
+                self.__buffer.fill_area(0, item_top + item_remain,
+                                        pw, self.__gapsize,
+                                        self.__bg_color)
 
-                    gap_offset += blocksize    
-                #end while
-            #end if
-        #end if
+
+            if (not self.__is_wrap_around()):
+                break
+                
+            item_y += self.__totalsize
+        #end while
 
         
     def move(self, nil, delta):
@@ -881,20 +1023,32 @@ class ImageStrip(Widget):
         x, y = 0, 0 #self.get_screen_pos()
         w, h = self.get_size()
         screen = self.get_screen()
-        
+
+        self.__fix_offset()
         if (not self.__is_wrap_around()):
             if (self.__offset + delta > self.__totalsize - h):
                 return
             elif (self.__offset + delta < 0):
                 return
         #end if
-
+        
         self.__offset += delta
+        """
+        if (not self.__is_wrap_around()):
+            print self.__offset, delta, self.__totalsize, h
+            if (self.__offset > self.__totalsize - h):
+                self.__offset = self.__totalsize - h
+                return
+            elif (self.__offset < 0):
+                self.__offset = 0
+                return
+        #end if
+
         while (self.__offset < 0):
             self.__offset += self.__totalsize
         while (self.__offset > self.__totalsize):
             self.__offset -= self.__totalsize
-            
+        """ 
 
         if (not self.may_render()): return
 
@@ -956,8 +1110,10 @@ class ImageStrip(Widget):
 
             # no need to have this animated while the widget is hidden
             if (not self.may_render()):
-                self.__offset = max(0, offset2)
+                #self.__offset = max(0, offset2)
+                self.__fix_offset()
                 self.__scroll_to_item_handler = None
+                self.invalidate_buffer()
                 return False
            
             # how far would we have to scroll up or down?
@@ -987,7 +1143,7 @@ class ImageStrip(Widget):
                 self.__scroll_to_item_handler = None
                 return False
 
-            delta = distance / 10
+            delta = distance / 3 #10
 
             if (distance < 2):
                 self.__move(0, max(-h, min(-1, delta)))
@@ -1014,6 +1170,11 @@ class ImageStrip(Widget):
             #self.set_events_blocked(True)
             #self.set_events_blocked(False)
 
+            # kill queued events
+            while (True):
+                e = gtk.gdk.event_get()
+                if (not e): break
+
 
     def fx_cycle_item(self, idx):
     
@@ -1047,40 +1208,31 @@ class ImageStrip(Widget):
         self.animate(50, fx, [0, item_w])
 
 
-    """
-    def fx_slide_left(self, wait = True):
-    
-        STEP = 16
-        x, y = self.get_screen_pos()
-        w, h = self.get_size()
-        screen = self.get_screen()
 
-        slider_width, nil = self.__scrollbar_pmap.get_size()
-        slider_width /= 2
-        w -= slider_width
+    def fx_slide_left(self):
+
+        def fx(params):
+            from_x, to_x = params
+            dx = (to_x - from_x) / 3
+            
+            if (dx > 0):
+                screen.move_area(x + dx, y, w - dx, h, -dx, 0)
+                screen.copy_pixmap(buf, from_x, 0, x + w - dx, y, dx, h)
+
+                params[0] = from_x + dx
+                params[1] = to_x
+                return True
 
-        buf = Pixmap(None, w, h) #x + w, y + h)
-        self.render_at(buf)
-        finished = threading.Event()
-        
-        def f(i):
-            screen.copy_pixmap(screen, x + STEP, y, x, y, w - STEP, h)
-            screen.copy_pixmap(buf, i, 0, x + w - STEP, y, STEP, h)
-            if (i < w - STEP):
-                gobject.timeout_add(5, f, i + STEP)
             else:
-                self.render()
-                finished.set()
-
-        self.set_events_blocked(True)                
-        f(0)        
-        while (wait and not finished.isSet()): gtk.main_iteration()
-        self.set_events_blocked(False)
+                dx = to_x - from_x
+                screen.move_area(x + dx, y, w - dx, h, -dx, 0)
+                screen.copy_pixmap(buf, from_x, 0, x + w - dx, y, dx, h)
+                
+                return False
 
 
-    def fx_slide_right(self, wait = True):
-    
-        STEP = 16
+        if (not self.may_render()): return
+
         x, y = self.get_screen_pos()
         w, h = self.get_size()
         screen = self.get_screen()
@@ -1090,26 +1242,56 @@ class ImageStrip(Widget):
             slider_width /= 2
             w -= slider_width
 
-        buf = Pixmap(None, w, h) #x + w, y + h)
+        buf = self.__buffer
         buf.fill_area(0, 0, w, h, self.__bg_color)
-        #self.render_at(buf)
-        finished = threading.Event()
-        
-        def f(i):
-            screen.copy_pixmap(screen, x, y, x + STEP, y, w - STEP, h)
-            screen.copy_pixmap(buf, w - i, 0, x, y, STEP, h)
-            if (i < w - STEP):
-                gobject.timeout_add(5, f, i + STEP)
-            else:
-                self.render()
-                finished.set()
+        self.render_at(buf)
 
-        self.set_events_blocked(True)                
-        f(0)
-        while (wait and not finished.isSet()): gtk.main_iteration(False)
-        self.set_events_blocked(False)
+        self.animate(50, fx, [0, w])
+
+
+
+    def fx_slide_right(self):
+
+        def fx(params):
+            from_x, to_x = params
+            dx = (to_x - from_x) / 3
             
+            if (dx > 0):
+                screen.move_area(x, y, w - dx, h, dx, 0)
+                screen.copy_pixmap(buf, w - from_x - dx, 0, x, y, dx, h)
+
+                params[0] = from_x + dx
+                params[1] = to_x
+                return True
+
+            else:
+                dx = to_x - from_x
+                screen.move_area(x, y, w - dx, h, dx, 0)
+                screen.copy_pixmap(buf, w - from_x - dx, 0, x, y, dx, h)
+
+                return False
+
+
+        if (not self.may_render()): return
+
+        x, y = self.get_screen_pos()
+        w, h = self.get_size()
+        screen = self.get_screen()
+
+        if (self.__scrollbar_pmap):
+            slider_width, nil = self.__scrollbar_pmap.get_size()
+            slider_width /= 2
+            w -= slider_width
+
+        buf = self.__buffer
+        buf.fill_area(0, 0, w, h, self.__bg_color)
+        self.render_at(buf)
+
+        self.animate(50, fx, [0, w])
+
+
             
+    """    
     def fx_slide_in(self, wait = True):
     
         STEP = 8
